@@ -424,11 +424,13 @@ through the code."
 		    (map (make-sparse-keymap)))
 		(define-key map [mouse-1] 'gds-show-last-stack)
 		(define-key map "\C-m" 'gds-show-last-stack)
-		(insert "[click here to show error stack]")
+		(insert "[click here (or RET) to show error stack]")
 		(add-text-properties beg (point)
 				     (list 'keymap map
 					   'mouse-face 'highlight))
-		(insert "\n")))
+		(insert "\n")
+		(add-text-properties (1- (point)) (point)
+				     (list 'keymap map))))
 	  (goto-char (point-min))
 	  (gds-associate-buffer client))
 	(pop-to-buffer buf)
