@@ -134,19 +134,6 @@
 
 /* #define GUILE_DEBUG_FREELIST */
 
-/* All the number support there is.
- */
-#define BIGNUMS
-
-/* GC should relinquish empty cons-pair arenas. */
-/* cmm:FIXME look at this after done mangling the GC */
-/* #define GC_FREE_SEGMENTS */
-
-/* Provide a scheme-accessible count-down timer that
- * generates a pseudo-interrupt.
- */
-#define TICKS
-
 
 /* Use engineering notation when converting numbers strings?
  */
@@ -410,6 +397,10 @@
 #define SCM_T_INTMAX_MIN  SCM_I_TYPE_MIN(scm_t_intmax,SCM_T_UINTMAX_MAX)
 #define SCM_T_INTMAX_MAX  SCM_I_TYPE_MAX(scm_t_intmax,SCM_T_UINTMAX_MAX)
 
+#define SCM_T_UINTPTR_MAX SCM_I_UTYPE_MAX(scm_t_uintptr)
+#define SCM_T_INTPTR_MIN  SCM_I_TYPE_MIN(scm_t_intptr,SCM_T_UINTPTR_MAX)
+#define SCM_T_INTPTR_MAX  SCM_I_TYPE_MAX(scm_t_intptr,SCM_T_UINTPTR_MAX)
+
 #define SCM_I_SIZE_MAX    SCM_I_UTYPE_MAX(size_t)
 #define SCM_I_SSIZE_MIN   SCM_I_TYPE_MIN(ssize_t,SCM_I_SIZE_MAX)
 #define SCM_I_SSIZE_MAX   SCM_I_TYPE_MAX(ssize_t,SCM_I_SIZE_MAX)
@@ -507,12 +498,24 @@ SCM_API void scm_async_tick (void);
 #ifdef BUILDING_LIBGUILE
 
 /* FIXME: should change names */
-# define SCM_ASYNC_TICK					\
-    do							\
-      {							\
-	if (SCM_I_CURRENT_THREAD->pending_asyncs)	\
-	  scm_async_click ();				\
-      }							\
+# define SCM_ASYNC_TICK                                                 \
+    do                                                                  \
+      {                                                                 \
+	if (SCM_UNLIKELY (SCM_I_CURRENT_THREAD->pending_asyncs))	\
+	  scm_async_click ();                                           \
+      }                                                                 \
+    while (0)
+
+/* SCM_ASYNC_TICK_WITH_CODE is only available to Guile itself */
+# define SCM_ASYNC_TICK_WITH_CODE(stmt)                                 \
+    do                                                                  \
+      {                                                                 \
+	if (SCM_UNLIKELY (SCM_I_CURRENT_THREAD->pending_asyncs))	\
+	  {                                                             \
+            stmt;                                                       \
+            scm_async_click ();                                         \
+          }                                                             \
+      }                                                                 \
     while (0)
 
 #else /* !BUILDING_LIBGUILE */
