@@ -52,6 +52,7 @@ SCM_SYMBOL (sym_size_t, "size_t");
 SCM_SYMBOL (sym_asterisk, "*");
 
 SCM_SYMBOL (sym_null, "%null-pointer");
+SCM_SYMBOL (sym_null_pointer_error, "null-pointer-error");
 
 /* The cell representing the null pointer.  */
 static const scm_t_bits null_pointer[2] =
@@ -60,6 +61,15 @@ static const scm_t_bits null_pointer[2] =
     0
   };
 
+/* Raise a null pointer dereference error.  */
+static void
+null_pointer_error (const char *func_name)
+{
+  scm_error (sym_null_pointer_error, func_name,
+	     "null pointer dereference", SCM_EOL, SCM_EOL);
+}
+
+
 static SCM cif_to_procedure (SCM cif, SCM func_ptr);
 
 
@@ -89,12 +99,8 @@ scm_take_foreign_pointer (scm_t_foreign_type type, void *ptr, size_t len,
                        | (finalizer ? (1<<16) : 0) | (len<<17));
   if (SCM_UNLIKELY ((word0 >> 17) != len))
     scm_out_of_range ("scm_take_foreign_pointer", scm_from_size_t (len));
-    
-  ret = PTR2SCM (scm_gc_malloc_pointerless (sizeof (scm_t_bits) * 2,
-                                            "foreign"));
-  SCM_SET_CELL_WORD_0 (ret, word0);
-  SCM_SET_CELL_WORD_1 (ret, (scm_t_bits)ptr);
 
+  ret = scm_cell (word0, (scm_t_bits) ptr);
   if (finalizer)
     {
       /* Register a finalizer for the newly created instance.  */
@@ -112,7 +118,7 @@ scm_take_foreign_pointer (scm_t_foreign_type type, void *ptr, size_t len,
 
 SCM_DEFINE (scm_foreign_ref, "foreign-ref", 1, 0, 0,
 	    (SCM foreign),
-	    "Reference the foreign value wrapped by @var{foreign}.\n\n"
+	    "Reference the foreign value pointed to by @var{foreign}.\n\n"
             "The value will be referenced according to its type.")
 #define FUNC_NAME s_scm_foreign_ref
 {
@@ -157,7 +163,7 @@ SCM_DEFINE (scm_foreign_ref, "foreign-ref", 1, 0, 0,
 
 SCM_DEFINE (scm_foreign_set_x, "foreign-set!", 2, 0, 0,
 	    (SCM foreign, SCM val),
-	    "Set the foreign value wrapped by @var{foreign}.\n\n"
+	    "Set the foreign value pointed to by @var{foreign}.\n\n"
             "The value will be set according to its type.")
 #define FUNC_NAME s_scm_foreign_set_x
 {
@@ -165,6 +171,12 @@ SCM_DEFINE (scm_foreign_set_x, "foreign-set!", 2, 0, 0,
   scm_t_uint8 *ptr;
 
   SCM_VALIDATE_FOREIGN (1, foreign);
+
+  if (SCM_UNLIKELY (scm_is_eq (foreign, PTR2SCM (&null_pointer))))
+    /* Attempting to modify the pointer value of NULL_POINTER (which is
+       read-only anyway), so raise an error.  */
+    null_pointer_error (FUNC_NAME);
+
   ptr = SCM_FOREIGN_POINTER (foreign, scm_t_uint8);
   ftype = SCM_FOREIGN_TYPE (foreign);
 
@@ -241,7 +253,7 @@ SCM_DEFINE (scm_foreign_to_bytevector, "foreign->bytevector", 1, 3, 0,
   ptr = SCM_FOREIGN_POINTER (foreign, scm_t_int8);
 
   if (SCM_UNLIKELY (ptr == NULL))
-    scm_misc_error (FUNC_NAME, "null pointer dereference", SCM_EOL);
+    null_pointer_error (FUNC_NAME);
 
   if (SCM_UNBNDP (uvec_type))
     btype = SCM_ARRAY_ELEMENT_TYPE_VU8;
@@ -426,7 +438,13 @@ scm_i_foreign_print (SCM foreign, SCM port, scm_print_state *pstate)
 
 #define ROUND_UP(len,align) (align?(((len-1)|(align-1))+1):len)
 
-SCM_DEFINE (scm_alignof, "alignof", 1, 0, 0, (SCM type), "")
+SCM_DEFINE (scm_alignof, "alignof", 1, 0, 0, (SCM type),
+            "Return the alignment of @var{type}, in bytes.\n\n"
+            "@var{type} should be a valid C type, like @code{int}.\n"
+            "Alternately @var{type} may be the symbol @code{*}, in which\n"
+            "case the alignment of a pointer is returned. @var{type} may\n"
+            "also be a list of types, in which case the alignment of a\n"
+            "@code{struct} with ABI-conventional packing is returned.")
 #define FUNC_NAME s_scm_alignof
 {
   if (SCM_I_INUMP (type))
@@ -468,7 +486,13 @@ SCM_DEFINE (scm_alignof, "alignof", 1, 0, 0, (SCM type), "")
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_sizeof, "sizeof", 1, 0, 0, (SCM type), "")
+SCM_DEFINE (scm_sizeof, "sizeof", 1, 0, 0, (SCM type),
+            "Return the size of @var{type}, in bytes.\n\n"
+            "@var{type} should be a valid C type, like @code{int}.\n"
+            "Alternately @var{type} may be the symbol @code{*}, in which\n"
+            "case the size of a pointer is returned. @var{type} may also\n"
+            "be a list of types, in which case the size of a @code{struct}\n"
+            "with ABI-conventional packing is returned.")
 #define FUNC_NAME s_scm_sizeof
 {
   if (SCM_I_INUMP (type))
@@ -634,7 +658,13 @@ fill_ffi_type (SCM type, ffi_type *ftype, ffi_type ***type_ptrs,
     
 SCM_DEFINE (scm_make_foreign_function, "make-foreign-function", 3, 0, 0,
             (SCM return_type, SCM func_ptr, SCM arg_types),
-            "foo")
+            "Make a foreign function.\n\n"
+            "Given the foreign void pointer @var{func_ptr}, its argument and\n"
+            "return types @var{arg_types} and @var{return_type}, return a\n"
+            "procedure that will pass arguments to the foreign function\n"
+            "and return appropriate values.\n\n"
+            "@var{arg_types} should be a list of foreign types.\n"
+            "@code{return_type} should be a foreign type.")
 #define FUNC_NAME s_scm_make_foreign_function
 {
   SCM walk, scm_cif;
@@ -844,134 +874,148 @@ cif_to_procedure (SCM cif, SCM func_ptr)
   return ret;
 }
 
+/* Set *LOC to the foreign representation of X with TYPE.  */
 static void
-unpack (ffi_type *type, void *loc, SCM x)
+unpack (const ffi_type *type, void *loc, SCM x)
 {
   switch (type->type)
     {
     case FFI_TYPE_FLOAT:
-      *(float*)loc = scm_to_double (x);
+      *(float *) loc = scm_to_double (x);
       break;
     case FFI_TYPE_DOUBLE:
-      *(double*)loc = scm_to_double (x);
+      *(double *) loc = scm_to_double (x);
       break;
     case FFI_TYPE_UINT8:
-      *(scm_t_uint8*)loc = scm_to_uint8 (x);
+      *(scm_t_uint8 *) loc = scm_to_uint8 (x);
       break;
     case FFI_TYPE_SINT8:
-      *(scm_t_int8*)loc = scm_to_int8 (x);
+      *(scm_t_int8 *) loc = scm_to_int8 (x);
       break;
     case FFI_TYPE_UINT16:
-      *(scm_t_uint16*)loc = scm_to_uint16 (x);
+      *(scm_t_uint16 *) loc = scm_to_uint16 (x);
       break;
     case FFI_TYPE_SINT16:
-      *(scm_t_int16*)loc = scm_to_int16 (x);
+      *(scm_t_int16 *) loc = scm_to_int16 (x);
       break;
     case FFI_TYPE_UINT32:
-      *(scm_t_uint32*)loc = scm_to_uint32 (x);
+      *(scm_t_uint32 *) loc = scm_to_uint32 (x);
       break;
     case FFI_TYPE_SINT32:
-      *(scm_t_int32*)loc = scm_to_int32 (x);
+      *(scm_t_int32 *) loc = scm_to_int32 (x);
       break;
     case FFI_TYPE_UINT64:
-      *(scm_t_uint64*)loc = scm_to_uint64 (x);
+      *(scm_t_uint64 *) loc = scm_to_uint64 (x);
       break;
     case FFI_TYPE_SINT64:
-      *(scm_t_int64*)loc = scm_to_int64 (x);
+      *(scm_t_int64 *) loc = scm_to_int64 (x);
       break;
     case FFI_TYPE_STRUCT:
       if (!SCM_FOREIGN_TYPED_P (x, VOID))
-        scm_wrong_type_arg_msg ("foreign-call", 0, x,
-                                "foreign void pointer");
+	scm_wrong_type_arg_msg ("foreign-call", 0, x, "foreign void pointer");
       if (SCM_FOREIGN_LEN (x) && SCM_FOREIGN_LEN (x) != type->size)
-        scm_wrong_type_arg_msg ("foreign-call", 0, x,
-                                "foreign void pointer of correct length");
+	scm_wrong_type_arg_msg ("foreign-call", 0, x,
+				"foreign void pointer of correct length");
       memcpy (loc, SCM_FOREIGN_POINTER (x, void), type->size);
       break;
     case FFI_TYPE_POINTER:
       if (!SCM_FOREIGN_TYPED_P (x, VOID))
-        scm_wrong_type_arg_msg ("foreign-call", 0, x,
-                                "foreign void pointer");
-      *(void**)loc = SCM_FOREIGN_POINTER (x, void);
+	scm_wrong_type_arg_msg ("foreign-call", 0, x, "foreign void pointer");
+      *(void **) loc = SCM_FOREIGN_POINTER (x, void);
       break;
     default:
       abort ();
     }
 }
 
+/* Return a Scheme representation of the foreign value at LOC of type TYPE.  */
 static SCM
-pack (ffi_type *type, void *loc)
+pack (const ffi_type * type, const void *loc)
 {
   switch (type->type)
     {
     case FFI_TYPE_VOID:
       return SCM_UNSPECIFIED;
     case FFI_TYPE_FLOAT:
-      return scm_from_double (*(float*)loc);
+      return scm_from_double (*(float *) loc);
     case FFI_TYPE_DOUBLE:
-      return scm_from_double (*(double*)loc);
+      return scm_from_double (*(double *) loc);
     case FFI_TYPE_UINT8:
-      return scm_from_uint8 (*(scm_t_uint8*)loc);
+      return scm_from_uint8 (*(scm_t_uint8 *) loc);
     case FFI_TYPE_SINT8:
-      return scm_from_int8 (*(scm_t_int8*)loc);
+      return scm_from_int8 (*(scm_t_int8 *) loc);
     case FFI_TYPE_UINT16:
-      return scm_from_uint16 (*(scm_t_uint16*)loc);
+      return scm_from_uint16 (*(scm_t_uint16 *) loc);
     case FFI_TYPE_SINT16:
-      return scm_from_int16 (*(scm_t_int16*)loc);
+      return scm_from_int16 (*(scm_t_int16 *) loc);
     case FFI_TYPE_UINT32:
-      return scm_from_uint32 (*(scm_t_uint32*)loc);
+      return scm_from_uint32 (*(scm_t_uint32 *) loc);
     case FFI_TYPE_SINT32:
-      return scm_from_int32 (*(scm_t_int32*)loc);
+      return scm_from_int32 (*(scm_t_int32 *) loc);
     case FFI_TYPE_UINT64:
-      return scm_from_uint64 (*(scm_t_uint64*)loc);
+      return scm_from_uint64 (*(scm_t_uint64 *) loc);
     case FFI_TYPE_SINT64:
-      return scm_from_int64 (*(scm_t_int64*)loc);
+      return scm_from_int64 (*(scm_t_int64 *) loc);
     case FFI_TYPE_STRUCT:
       {
-        void *mem = scm_gc_malloc_pointerless (type->size, "foreign");
-        memcpy (mem, loc, type->size);
-        return scm_take_foreign_pointer (SCM_FOREIGN_TYPE_VOID,
-                                         mem, type->size, NULL);
+	void *mem = scm_gc_malloc_pointerless (type->size, "foreign");
+	memcpy (mem, loc, type->size);
+	return scm_take_foreign_pointer (SCM_FOREIGN_TYPE_VOID,
+					 mem, type->size, NULL);
       }
     case FFI_TYPE_POINTER:
       return scm_take_foreign_pointer (SCM_FOREIGN_TYPE_VOID,
-                                       *(void**)loc, 0, NULL);
+				       *(void **) loc, 0, NULL);
     default:
       abort ();
     }
 }
 
+
 SCM
-scm_i_foreign_call (SCM foreign, SCM *argv)
+scm_i_foreign_call (SCM foreign, const SCM *argv)
 {
   /* FOREIGN is the pair that cif_to_procedure set as the 0th element of the
      objtable. */
   ffi_cif *cif;
-  void (*func)();
+  void (*func) (void);
   scm_t_uint8 *data;
   void *rvalue;
   void **args;
   unsigned i;
+  size_t arg_size;
   scm_t_ptrdiff off;
 
-  cif = SCM_FOREIGN_POINTER (scm_car (foreign), ffi_cif);
-  func = SCM_FOREIGN_POINTER (scm_cdr (foreign), void);
-  
-  /* arg pointers */
-  args = alloca (sizeof(void*) * cif->nargs);
-  /* arg values, then return type value */
-  data = alloca (ROUND_UP (cif->bytes, cif->rtype->alignment)
-                 + cif->rtype->size);
-  /* unpack argv to native values, setting argv pointers */
-  off = 0;
-  for (i = 0; i < cif->nargs; i++)
+  cif = SCM_FOREIGN_POINTER (SCM_CAR (foreign), ffi_cif);
+  func = SCM_FOREIGN_POINTER (SCM_CDR (foreign), void);
+
+  /* Argument pointers.  */
+  args = alloca (sizeof (void *) * cif->nargs);
+
+  /* Compute the amount of memory needed to store all the argument values.
+     Note: as of libffi 3.0.9 `cif->bytes' is undocumented and is zero, so it
+     can't be used for that purpose.  */
+  for (i = 0, arg_size = 0;
+       i < cif->nargs;
+       i++, arg_size)
+    arg_size += ROUND_UP (cif->arg_types[i]->size,
+			  cif->arg_types[i]->alignment);
+
+  /* Space for argument values, followed by return value.  */
+  data = alloca (arg_size
+		 + ROUND_UP (cif->rtype->size, cif->rtype->alignment));
+
+  /* Unpack ARGV to native values, setting ARGV pointers.  */
+  for (i = 0, off = 0;
+       i < cif->nargs;
+       off += cif->arg_types[i]->size, i++)
     {
       off = ROUND_UP (off, cif->arg_types[i]->alignment);
       args[i] = data + off;
       unpack (cif->arg_types[i], args[i], argv[i]);
-      off += cif->arg_types[i]->size;
     }
-  /* prep space for the return value */
+
+  /* Prepare space for the return value.  */
   off = ROUND_UP (off, cif->rtype->alignment);
   rvalue = data + off;
 
