@@ -1,4 +1,4 @@
-/* Copyright (C) 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+/* Copyright (C) 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -59,10 +59,16 @@
 
 #include "libguile/posix.h"  /* for `scm_i_locale_mutex' */
 
-#if (defined HAVE_LANGINFO_H) && (defined HAVE_NL_TYPES_H)
+#ifdef HAVE_LANGINFO_H
 # include <langinfo.h>
+#endif
+#ifdef HAVE_NL_TYPES_H
 # include <nl_types.h>
 #endif
+#ifndef HAVE_NL_ITEM
+/* Cygwin has <langinfo.h> but lacks <nl_types.h> and `nl_item'.  */
+typedef int nl_item;
+#endif
 
 #ifndef HAVE_SETLOCALE
 static inline char *
@@ -1102,7 +1108,9 @@ static SCM
 chr_to_case (SCM chr, scm_t_locale c_locale, 
 	     scm_t_uint32 *(*func) (const scm_t_uint32 *, size_t, const char *,
 				    uninorm_t, scm_t_uint32 *, size_t *),
+	     const char *func_name,
 	     int *err)
+#define FUNC_NAME func_name
 {
   int ret;
   scm_t_wchar *buf;
@@ -1137,6 +1145,7 @@ chr_to_case (SCM chr, scm_t_locale c_locale,
 
   return convchar;
 }
+#undef FUNC_NAME
 
 SCM_DEFINE (scm_char_locale_downcase, "char-locale-downcase", 1, 1, 0,
 	    (SCM chr, SCM locale),
@@ -1151,7 +1160,7 @@ SCM_DEFINE (scm_char_locale_downcase, "char-locale-downcase", 1, 1, 0,
   SCM_VALIDATE_CHAR (1, chr);
   SCM_VALIDATE_OPTIONAL_LOCALE_COPY (2, locale, c_locale);
 
-  ret = chr_to_case (chr, c_locale, u32_tolower, &err);
+  ret = chr_to_case (chr, c_locale, u32_tolower, FUNC_NAME, &err);
 
   if (err != 0)
     {
@@ -1175,7 +1184,7 @@ SCM_DEFINE (scm_char_locale_upcase, "char-locale-upcase", 1, 1, 0,
   SCM_VALIDATE_CHAR (1, chr);
   SCM_VALIDATE_OPTIONAL_LOCALE_COPY (2, locale, c_locale);
 
-  ret = chr_to_case (chr, c_locale, u32_toupper, &err);
+  ret = chr_to_case (chr, c_locale, u32_toupper, FUNC_NAME, &err);
 
   if (err != 0)
     {
@@ -1199,7 +1208,7 @@ SCM_DEFINE (scm_char_locale_titlecase, "char-locale-titlecase", 1, 1, 0,
   SCM_VALIDATE_CHAR (1, chr);
   SCM_VALIDATE_OPTIONAL_LOCALE_COPY (2, locale, c_locale);
 
-  ret = chr_to_case (chr, c_locale, u32_totitle, &err);
+  ret = chr_to_case (chr, c_locale, u32_totitle, FUNC_NAME, &err);
 
   if (err != 0)
     {
@@ -1214,7 +1223,9 @@ static SCM
 str_to_case (SCM str, scm_t_locale c_locale,
 	     scm_t_uint32 *(*func) (const scm_t_uint32 *, size_t, const char *,
 				    uninorm_t, scm_t_uint32 *, size_t *),
+	     const char *func_name,
 	     int *err)
+#define FUNC_NAME func_name
 {
   scm_t_wchar *c_str, *c_buf;
   scm_t_uint32 *c_convstr;
@@ -1253,6 +1264,7 @@ str_to_case (SCM str, scm_t_locale c_locale,
 
   return convstr;
 }
+#undef FUNC_NAME
 
 SCM_DEFINE (scm_string_locale_upcase, "string-locale-upcase", 1, 1, 0,
 	    (SCM str, SCM locale),
@@ -1268,7 +1280,7 @@ SCM_DEFINE (scm_string_locale_upcase, "string-locale-upcase", 1, 1, 0,
   SCM_VALIDATE_STRING (1, str);
   SCM_VALIDATE_OPTIONAL_LOCALE_COPY (2, locale, c_locale);
 
-  ret = str_to_case (str, c_locale, u32_toupper, &err);
+  ret = str_to_case (str, c_locale, u32_toupper, FUNC_NAME, &err);
   
   if (err != 0)
     {
@@ -1293,7 +1305,7 @@ SCM_DEFINE (scm_string_locale_downcase, "string-locale-downcase", 1, 1, 0,
   SCM_VALIDATE_STRING (1, str);
   SCM_VALIDATE_OPTIONAL_LOCALE_COPY (2, locale, c_locale);
 
-  ret = str_to_case (str, c_locale, u32_tolower, &err);
+  ret = str_to_case (str, c_locale, u32_tolower, FUNC_NAME, &err);
 
   if (err != 0)
     {
@@ -1318,7 +1330,7 @@ SCM_DEFINE (scm_string_locale_titlecase, "string-locale-titlecase", 1, 1, 0,
   SCM_VALIDATE_STRING (1, str);
   SCM_VALIDATE_OPTIONAL_LOCALE_COPY (2, locale, c_locale);
 
-  ret = str_to_case (str, c_locale, u32_totitle, &err);
+  ret = str_to_case (str, c_locale, u32_totitle, FUNC_NAME, &err);
 
   if (err != 0)
     {
@@ -1453,7 +1465,10 @@ SCM_DEFINE (scm_locale_string_to_inexact, "locale-string->inexact",
       setting of the current locale.  If nl_langinfo supports CODESET,
       we can convert the string properly using scm_from_stringn.  If
       CODESET is not supported, we won't be able to make much sense of
-      the returned string. */
+      the returned string.
+
+   Note: We don't use Gnulib's `nl_langinfo' module because it's currently not
+   as complete as the compatibility hacks in `i18n.scm'.  */
 
 
 SCM_DEFINE (scm_nl_langinfo, "nl-langinfo", 1, 1, 0,
@@ -1851,7 +1866,8 @@ scm_init_i18n ()
 void
 scm_bootstrap_i18n ()
 {
-  scm_c_register_extension ("libguile", "scm_init_i18n",
+  scm_c_register_extension ("libguile-" SCM_EFFECTIVE_VERSION,
+                            "scm_init_i18n",
 			    (scm_t_extension_init_func) scm_init_i18n,
 			    NULL);
 
