@@ -74,7 +74,6 @@
 #include "libguile/iselect.h"
 #include "libguile/ioext.h"
 #include "libguile/keywords.h"
-#include "libguile/lang.h"
 #include "libguile/list.h"
 #include "libguile/load.h"
 #include "libguile/macros.h"
@@ -557,9 +556,6 @@ scm_i_init_guile (SCM_STACKITEM *base)
   scm_init_simpos ();
   scm_init_dynamic_linking (); /* Requires smob_prehistory */
   scm_bootstrap_i18n ();
-#if SCM_ENABLE_ELISP
-  scm_init_lang ();
-#endif /* SCM_ENABLE_ELISP */
   scm_init_script ();
 
   scm_init_goops ();
@@ -586,6 +582,7 @@ scm_i_init_guile (SCM_STACKITEM *base)
 
   atexit (cleanup_for_exit);
   scm_load_startup_files ();
+  scm_init_load_should_autocompile ();
 }
 
 /*
