@@ -27,6 +27,7 @@
 #include <stdio.h>
 
 #include "libguile/_scm.h"
+#include "libguile/private-gc.h" /* scm_getenv_int */
 #include "libguile/libpath.h"
 #include "libguile/fports.h"
 #include "libguile/read.h"
@@ -37,7 +38,6 @@
 #include "libguile/root.h"
 #include "libguile/strings.h"
 #include "libguile/modules.h"
-#include "libguile/lang.h"
 #include "libguile/chars.h"
 #include "libguile/srfi-13.h"
 
@@ -900,7 +900,6 @@ scm_init_load ()
 
   scm_loc_compile_fallback_path
     = SCM_VARIABLE_LOC (scm_c_define ("%compile-fallback-path", SCM_BOOL_F));
-
   scm_loc_load_should_autocompile
     = SCM_VARIABLE_LOC (scm_c_define ("%load-should-autocompile", SCM_BOOL_F));
 
@@ -917,6 +916,15 @@ scm_init_load ()
 #include "libguile/load.x"
 }
 
+void
+scm_init_load_should_autocompile ()
+{
+  *scm_loc_load_should_autocompile =
+    scm_from_bool (scm_getenv_int ("GUILE_AUTO_COMPILE", 1));
+}
+  
+  
+
 /*
   Local Variables:
   c-file-style: "gnu"
