@@ -91,20 +91,11 @@ SCM_API scm_t_option scm_read_opts[];
 #define SCM_RECORD_POSITIONS_P scm_read_opts[1].val
 #define SCM_CASE_INSENSITIVE_P scm_read_opts[2].val
 #define SCM_KEYWORD_STYLE      scm_read_opts[3].val
-#if SCM_ENABLE_ELISP
 #define SCM_ELISP_VECTORS_P    scm_read_opts[4].val
 #define SCM_ESCAPED_PARENS_P   scm_read_opts[5].val
 #define SCM_R6RS_ESCAPES_P     scm_read_opts[6].val
 #define SCM_SQUARE_BRACKETS_P  scm_read_opts[7].val
-#else
-#define SCM_R6RS_ESCAPES_P     scm_read_opts[4].val
-#define SCM_SQUARE_BRACKETS_P  scm_read_opts[5].val
-#endif
 
-#if SCM_ENABLE_ELISP
 #define SCM_N_READ_OPTIONS 8
-#else
-#define SCM_N_READ_OPTIONS 6
-#endif
 
 #endif  /* PRIVATE_OPTIONS */ 
