@@ -29,7 +29,6 @@
 #include <unistd.h>
 #endif
 #include <stdio.h>
-#include <assert.h>
 
 #ifdef HAVE_STRING_H
 #include <string.h>   /* for memset used by FD_ZERO on Solaris 10 */
@@ -338,7 +337,6 @@ guilify_self_1 (SCM_STACKITEM *base)
   t->block_asyncs = 1;
   t->pending_asyncs = 1;
   t->critical_section_level = 0;
-  t->last_debug_frame = NULL;
   t->base = base;
 #ifdef __ia64__
   /* Calculate and store off the base of this thread's register
@@ -1948,15 +1946,14 @@ scm_init_threads ()
   guilify_self_2 (SCM_BOOL_F);
   threads_initialized_p = 1;
 
-  dynwind_critical_section_mutex =
-    scm_permanent_object (scm_make_recursive_mutex ());
+  dynwind_critical_section_mutex = scm_make_recursive_mutex ();
 }
 
 void
 scm_init_threads_default_dynamic_state ()
 {
   SCM state = scm_make_dynamic_state (scm_current_dynamic_state ());
-  scm_i_default_dynamic_state = scm_permanent_object (state);
+  scm_i_default_dynamic_state = state;
 }
 
 void
