@@ -32,13 +32,13 @@
 VM_DEFINE_FUNCTION (128, not, "not", 1)
 {
   ARGS1 (x);
-  RETURN (scm_from_bool (scm_is_false_or_nil (x)));
+  RETURN (scm_from_bool (scm_is_false (x)));
 }
 
 VM_DEFINE_FUNCTION (129, not_not, "not-not", 1)
 {
   ARGS1 (x);
-  RETURN (scm_from_bool (!scm_is_false_or_nil (x)));
+  RETURN (scm_from_bool (!scm_is_false (x)));
 }
 
 VM_DEFINE_FUNCTION (130, eq, "eq?", 2)
@@ -56,13 +56,13 @@ VM_DEFINE_FUNCTION (131, not_eq, "not-eq?", 2)
 VM_DEFINE_FUNCTION (132, nullp, "null?", 1)
 {
   ARGS1 (x);
-  RETURN (scm_from_bool (scm_is_null_or_nil (x)));
+  RETURN (scm_from_bool (scm_is_null (x)));
 }
 
 VM_DEFINE_FUNCTION (133, not_nullp, "not-null?", 1)
 {
   ARGS1 (x);
-  RETURN (scm_from_bool (!scm_is_null_or_nil (x)));
+  RETURN (scm_from_bool (!scm_is_null (x)));
 }
 
 VM_DEFINE_FUNCTION (134, eqv, "eqv?", 2)
@@ -287,10 +287,23 @@ VM_DEFINE_FUNCTION (157, ash, "ash", 2)
   if (SCM_I_INUMP (x) && SCM_I_INUMP (y))
     {
       if (SCM_I_INUM (y) < 0)
+        /* Right shift, will be a fixnum. */
         RETURN (SCM_I_MAKINUM (SCM_I_INUM (x) >> -SCM_I_INUM (y)));
-      else if ((SCM_I_INUM (x) << SCM_I_INUM (y)) >> SCM_I_INUM (y)
-               == SCM_I_INUM (x))
-        RETURN (SCM_I_MAKINUM (SCM_I_INUM (x) << SCM_I_INUM (y)));
+      else
+        /* Left shift. See comments in scm_ash. */
+        {
+          long nn, bits_to_shift;
+
+          nn = SCM_I_INUM (x);
+          bits_to_shift = SCM_I_INUM (y);
+
+          if (bits_to_shift < SCM_I_FIXNUM_BIT-1
+              && ((unsigned long)
+                  (SCM_SRS (nn, (SCM_I_FIXNUM_BIT-1 - bits_to_shift)) + 1)
+                  <= 1))
+            RETURN (SCM_I_MAKINUM (nn << bits_to_shift));
+          /* fall through */
+        }
       /* fall through */
     }
   SYNC_REGISTER ();
@@ -333,7 +346,7 @@ VM_DEFINE_FUNCTION (161, vector_ref, "vector-ref", 2)
 {
   long i = 0;
   ARGS2 (vect, idx);
-  if (SCM_LIKELY (SCM_I_IS_VECTOR (vect)
+  if (SCM_LIKELY (SCM_I_IS_NONWEAK_VECTOR (vect)
                   && SCM_I_INUMP (idx)
                   && ((i = SCM_I_INUM (idx)) >= 0)
                   && i < SCM_I_VECTOR_LENGTH (vect)))
@@ -350,7 +363,7 @@ VM_DEFINE_INSTRUCTION (162, vector_set, "vector-set", 0, 3, 0)
   long i = 0;
   SCM vect, idx, val;
   POP (val); POP (idx); POP (vect);
-  if (SCM_LIKELY (SCM_I_IS_VECTOR (vect)
+  if (SCM_LIKELY (SCM_I_IS_NONWEAK_VECTOR (vect)
                   && SCM_I_INUMP (idx)
                   && ((i = SCM_I_INUM (idx)) >= 0)
                   && i < SCM_I_VECTOR_LENGTH (vect)))
@@ -502,7 +515,10 @@ VM_DEFINE_FUNCTION (168, struct_set, "struct-set", 3)
 VM_DEFINE_FUNCTION (169, class_of, "class-of", 1)
 {
   ARGS1 (obj);
-  RETURN (SCM_INSTANCEP (obj) ? SCM_CLASS_OF (obj) : scm_class_of (obj));
+  if (SCM_INSTANCEP (obj))
+    RETURN (SCM_CLASS_OF (obj));
+  SYNC_REGISTER ();
+  RETURN (scm_class_of (obj));
 }
 
 VM_DEFINE_FUNCTION (170, slot_ref, "slot-ref", 2)
