@@ -33,6 +33,8 @@
     call-with-values @call-with-values
     call-with-current-continuation @call-with-current-continuation
     call/cc
+    dynamic-wind
+    @dynamic-wind
     values
     eq? eqv? equal?
     memq memv
@@ -56,9 +58,14 @@
 
     vector-ref vector-set!
     variable-ref variable-set!
+    variable-bound?
     ;; args of variable-set are switched; it needs special help
 
-    struct? struct-vtable make-struct
+    fluid-ref fluid-set!
+
+    @prompt call-with-prompt @abort abort-to-prompt
+
+    struct? struct-vtable make-struct struct-ref struct-set!
 
     bytevector-u8-ref bytevector-u8-set!
     bytevector-s8-ref bytevector-s8-set!
@@ -112,7 +119,7 @@
     caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr
     cdaaar cdaadr cdadar cdaddr cddaar cddadr cdddar cddddr
     vector-ref
-    struct? struct-vtable make-struct
+    struct? struct-vtable make-struct struct-ref
     bytevector-u8-ref bytevector-s8-ref
     bytevector-u16-ref bytevector-u16-native-ref
     bytevector-s16-ref bytevector-s16-native-ref
@@ -177,13 +184,15 @@
             ((symbol? in) `(cons* ,@(reverse out) ,in))
             ((pair? (car in))
              (lp (cdr in)
-                 (cons `(make-application src (make-primitive-ref src ',(caar in))
-                                          ,(inline-args (cdar in)))
+                 (cons (if (eq? (caar in) 'quote)
+                           `(make-const src ,@(cdar in))
+                           `(make-application src (make-primitive-ref src ',(caar in))
+                                              ,(inline-args (cdar in))))
                        out)))
             ((symbol? (car in))
              ;; assume it's locally bound
              (lp (cdr in) (cons (car in) out)))
-            ((number? (car in))
+            ((self-evaluating? (car in))
              (lp (cdr in) (cons `(make-const src ,(car in)) out)))
             (else
              (error "what what" (car in))))))
@@ -361,3 +370,120 @@
   (bytevector-ieee-double-native-ref vec (* i 8)))
 (define-primitive-expander f64vector-set! (vec i x)
   (bytevector-ieee-double-native-set! vec (* i 8) x))
+
+(hashq-set! *primitive-expand-table*
+            'dynamic-wind
+            (case-lambda
+              ((src pre thunk post)
+               ;; Here we will make concessions to the fact that our inliner is
+               ;; lame, and add a hack.
+               (cond
+                ((lambda? thunk)
+                 (let ((PRE (gensym " pre"))
+                       (POST (gensym " post")))
+                   (make-let
+                    src
+                    '(pre post)
+                    (list PRE POST)
+                    (list pre post)
+                    (make-dynwind
+                     src
+                     (make-lexical-ref #f 'pre PRE)
+                     (make-application #f thunk '())
+                     (make-lexical-ref #f 'post POST)))))
+                (else
+                 (let ((PRE (gensym " pre"))
+                       (THUNK (gensym " thunk"))
+                       (POST (gensym " post")))
+                   (make-let
+                    src
+                    '(pre thunk post)
+                    (list PRE THUNK POST)
+                    (list pre thunk post)
+                    (make-dynwind
+                     src
+                     (make-lexical-ref #f 'pre PRE)
+                     (make-application #f (make-lexical-ref #f 'thunk THUNK) '())
+                     (make-lexical-ref #f 'post POST)))))))
+              (else #f)))
+
+(hashq-set! *primitive-expand-table*
+            '@dynamic-wind
+            (case-lambda
+              ((src pre expr post)
+               (let ((PRE (gensym " pre"))
+                     (POST (gensym " post")))
+                 (make-let
+                  src
+                  '(pre post)
+                  (list PRE POST)
+                  (list pre post)
+                  (make-dynwind
+                   src
+                   (make-lexical-ref #f 'pre PRE)
+                   expr
+                   (make-lexical-ref #f 'post POST)))))))
+
+(hashq-set! *primitive-expand-table*
+            'fluid-ref
+            (case-lambda
+              ((src fluid) (make-dynref src fluid))
+              (else #f)))
+
+(hashq-set! *primitive-expand-table*
+            'fluid-set!
+            (case-lambda
+              ((src fluid exp) (make-dynset src fluid exp))
+              (else #f)))
+
+(hashq-set! *primitive-expand-table*
+            '@prompt
+            (case-lambda
+              ((src tag exp handler)
+               (let ((args-sym (gensym)))
+                 (make-prompt
+                  src tag exp
+                  ;; If handler itself is a lambda, the inliner can do some
+                  ;; trickery here.
+                  (make-lambda-case
+                   (tree-il-src handler) '() #f 'args #f '() (list args-sym)
+                   (make-application #f (make-primitive-ref #f 'apply)
+                                     (list handler
+                                           (make-lexical-ref #f 'args args-sym)))
+                   #f))))
+              (else #f)))
+
+(hashq-set! *primitive-expand-table*
+            'call-with-prompt
+            (case-lambda
+              ((src tag thunk handler)
+               ;; Sigh. Until the inliner does its job, manually inline
+               ;; (let ((h (lambda ...))) (prompt k x h))
+               (cond
+                ((lambda? handler)
+                 (let ((args-sym (gensym)))
+                   (make-prompt
+                    src tag (make-application #f thunk '())
+                    ;; If handler itself is a lambda, the inliner can do some
+                    ;; trickery here.
+                    (make-lambda-case
+                     (tree-il-src handler) '() #f 'args #f '() (list args-sym)
+                     (make-application #f (make-primitive-ref #f 'apply)
+                                       (list handler
+                                             (make-lexical-ref #f 'args args-sym)))
+                     #f))))
+                (else #f)))
+              (else #f)))
+
+(hashq-set! *primitive-expand-table*
+            '@abort
+            (case-lambda
+              ((src tag tail-args)
+               (make-abort src tag '() tail-args))
+              (else #f)))
+(hashq-set! *primitive-expand-table*
+            'abort-to-prompt
+            (case-lambda
+              ((src tag . args)
+               (make-abort src tag args (make-const #f '())))
+              (else #f)))
